@@ -13,7 +13,7 @@
  * You should have received a copy of the GNU General Public License
  * along with the hkl library.  If not, see <http://www.gnu.org/licenses/>.
  *
- * Copyright (C) 2003-2023 Synchrotron SOLEIL
+ * Copyright (C) 2003-2024 Synchrotron SOLEIL
  *                         L'Orme des Merisiers Saint-Aubin
  *                         BP 48 91192 GIF-sur-YVETTE CEDEX
  *
@@ -25,6 +25,8 @@
 #ifdef DEBUG
 # define CGLM_DEFINE_PRINTS
 # define CGLM_PRINT_PRECISION 7
+#else
+# define NDEBUG
 #endif
 
 #include <stdio.h>
@@ -378,183 +380,254 @@ HKL_BINOCULARS_SPACE_ANGLES_IMPL(uint32_t);
 
 /* qcustom */
 
+struct _HklBinocularsProjectionAxis
+{
+        const char *name;
+        const char *description;
+};
+
+typedef struct _HklBinocularsProjectionAxis HklBinocularsProjectionAxis;
+
+static const HklBinocularsProjectionAxis azimuth =
+{
+        .name = "azimuth",
+        .description = "The azimuthal angle in the yz plan of the kf vector",
+};
+static const HklBinocularsProjectionAxis deltalab =
+{
+        .name = "deltalab",
+        .description = "The delta horizontal deviation in the lab basis",
+};
+static const HklBinocularsProjectionAxis gammalab =
+{
+        .name = "gammalab",
+        .description = "The gamma elevation in the lab basis",
+};
+static const HklBinocularsProjectionAxis q =
+{
+        .name = "q",
+        .description = "The norm of the q vector",
+};
+static const HklBinocularsProjectionAxis qpar =
+{
+        .name = "qpar",
+        .description = "The norm of the q vector projected onto the plan parallel to the surface."
+};
+static const HklBinocularsProjectionAxis qpars =
+{
+        .name = "qpars",
+        .description = "The signed norm of the q vector projected onto the plan parallel to the surface."
+};
+static const HklBinocularsProjectionAxis qper =
+{
+        .name = "qper",
+        .description = "The norm of the q vector projected onto the surface vector (perpendicular to the surface)."
+};
+static const HklBinocularsProjectionAxis qx =
+{
+        .name = "qx",
+        .description = "The x coordinate of the Q vector in the sample basis."
+};
+static const HklBinocularsProjectionAxis qy =
+{
+        .name = "qy",
+        .description = "The y coordinate of the Q vector in the sample basis."
+};
+static const HklBinocularsProjectionAxis qz =
+{
+        .name = "qz",
+        .description = "The z coordinate of the Q vector in the sample basis."
+};
+static const HklBinocularsProjectionAxis phi =
+{
+        .name = "phi",
+        .description = "The azimuthal angle in the yz plan of the q vector",
+};
+static const HklBinocularsProjectionAxis sampleaxis =
+{
+        .name = "sampleaxis",
+        .description = "The name of a sample axis",
+};
+static const HklBinocularsProjectionAxis timestamp =
+{
+        .name = "timestamp",
+        .description = "The timestamp of each image",
+};
+static const HklBinocularsProjectionAxis timescan0 =
+{
+        .name = "timescan0",
+        .description = "The timestamp of the first image of a scan",
+};
+static const HklBinocularsProjectionAxis tth =
+{
+        .name = "tth",
+        .description = "The 2 * theta angle.",
+};
+static const HklBinocularsProjectionAxis xp =
+{
+        .name = "xp",
+        .description = "The x coordinate of pixels projected in the yz plan",
+};
+static const HklBinocularsProjectionAxis yp =
+{
+        .name="yp",
+        .description = "The y coordinate of pixels projected in the yz plan",
+};
+static const HklBinocularsProjectionAxis x =
+{
+        .name="x",
+        .description = "The x coordinate of pixels",
+};
+static const HklBinocularsProjectionAxis y =
+{
+        .name="y",
+        .description = "The y coordinate of pixels",
+};
+static const HklBinocularsProjectionAxis z =
+{
+        .name="z",
+        .description = "The z coordinate of pixels",
+};
+
+
+#define PROJECTION(...) do {                                            \
+                int i;                                                  \
+                int n;                                                  \
+                static const HklBinocularsProjectionAxis axes[] = {__VA_ARGS__}; \
+                static const char *axes_names[ARRAY_SIZE(axes)];        \
+                                                                        \
+                n = ARRAY_SIZE(axes);                                   \
+                assert(n <= darray_size(space->axes));                  \
+                assert(n <= n_resolutions);                             \
+                                                                        \
+                for(i=0; i<n; ++i)                                      \
+                        axes_names[i] = axes[i].name;                   \
+                names = axes_names;                                     \
+        }while(0)
+
 static const char **axis_name_from_subprojection(HklBinocularsQCustomSubProjectionEnum subprojection,
                                                  HklBinocularsSpace *space,
                                                  int n_resolutions)
 {
-        const char **names;
+        const char **names = NULL;
 
         switch(subprojection){
         case HKL_BINOCULARS_QCUSTOM_SUB_PROJECTION_QX_QY_QZ:
         case HKL_BINOCULARS_QCUSTOM_NUM_SUBPROJECTIONS:
         {
-                static const char *names_qx_qy_qz[] = {"qx", "qy", "qz"};
-                assert(ARRAY_SIZE(names_qx_qy_qz) == darray_size(space->axes));
-                assert(ARRAY_SIZE(names_qx_qy_qz) == n_resolutions);
-                names = names_qx_qy_qz;
+                PROJECTION(qx, qy, qz);
                 break;
         }
         case HKL_BINOCULARS_QCUSTOM_SUB_PROJECTION_Q_TTH_TIMESTAMP:
         {
-                static const char *names_q_tth_timestamp[] = {"q", "tth", "timestamp"};
-                assert(ARRAY_SIZE(names_q_tth_timestamp) == darray_size(space->axes));
-                assert(ARRAY_SIZE(names_q_tth_timestamp) == n_resolutions);
-                names = names_q_tth_timestamp;
+                PROJECTION(q, tth, timestamp);
                 break;
         }
         case HKL_BINOCULARS_QCUSTOM_SUB_PROJECTION_Q_TIMESTAMP:
         {
-                static const char *names_q_timestamp[] = {"q", "timestamp"};
-                assert(ARRAY_SIZE(names_q_timestamp) <= darray_size(space->axes));
-                assert(ARRAY_SIZE(names_q_timestamp) <= n_resolutions);
-                names = names_q_timestamp;
+                PROJECTION(q, timestamp);
                 break;
         }
         case HKL_BINOCULARS_QCUSTOM_SUB_PROJECTION_QPAR_QPER_TIMESTAMP:
         {
-                static const char *names_qpar_qper_timestamp[] = {"qpar", "qper", "timestamp"};
-                assert(ARRAY_SIZE(names_qpar_qper_timestamp) == darray_size(space->axes));
-                assert(ARRAY_SIZE(names_qpar_qper_timestamp) == n_resolutions);
-                names = names_qpar_qper_timestamp;
+                PROJECTION(qpar, qper, timestamp);
                 break;
         }
         case HKL_BINOCULARS_QCUSTOM_SUB_PROJECTION_QPAR_QPER:
         {
-                static const char *names_qpar_qper[] = {"qpar", "qper"};
-                assert(ARRAY_SIZE(names_qpar_qper) <= darray_size(space->axes));
-                assert(ARRAY_SIZE(names_qpar_qper) <= n_resolutions);
-                names = names_qpar_qper;
+                PROJECTION(qpar, qper);
                 break;
         }
         case HKL_BINOCULARS_QCUSTOM_SUB_PROJECTION_Q_PHI_QX:
         {
-                static const char *names_q_phi_qx[] = {"q", "phi", "qx"};
-                assert(ARRAY_SIZE(names_q_phi_qx) == darray_size(space->axes));
-                assert(ARRAY_SIZE(names_q_phi_qx) == n_resolutions);
-                names = names_q_phi_qx;
+                PROJECTION(q, phi, qx);
                 break;
         }
         case HKL_BINOCULARS_QCUSTOM_SUB_PROJECTION_Q_PHI_QY:
         {
-                static const char *names_q_phi_qy[] = {"q", "phi", "qy"};
-                assert(ARRAY_SIZE(names_q_phi_qy) == darray_size(space->axes));
-                assert(ARRAY_SIZE(names_q_phi_qy) == n_resolutions);
-                names = names_q_phi_qy;
+                PROJECTION(q, phi, qy);
                 break;
         }
         case HKL_BINOCULARS_QCUSTOM_SUB_PROJECTION_Q_PHI_QZ:
         {
-                static const char *names_q_phi_qz[] = {"q", "phi", "qz"};
-                assert(ARRAY_SIZE(names_q_phi_qz) == darray_size(space->axes));
-                assert(ARRAY_SIZE(names_q_phi_qz) == n_resolutions);
-                names = names_q_phi_qz;
+                PROJECTION(q, phi, qz);
                 break;
         }
         case HKL_BINOCULARS_QCUSTOM_SUB_PROJECTION_Q_STEREO:
         {
-                static const char *names_q_stereo[] = {"q", "xp", "yp"};
-                assert(ARRAY_SIZE(names_q_stereo) == darray_size(space->axes));
-                assert(ARRAY_SIZE(names_q_stereo) == n_resolutions);
-                names = names_q_stereo;
+                PROJECTION(q, xp, yp);
                 break;
         }
         case HKL_BINOCULARS_QCUSTOM_SUB_PROJECTION_DELTALAB_GAMMALAB_SAMPLEAXIS:
         {
-                static const char *names_deltalab_gammalab_sampleaxis[] = {"deltalab", "gammalab", "sampleaxis"};
-                assert(ARRAY_SIZE(names_deltalab_gammalab_sampleaxis) == darray_size(space->axes));
-                assert(ARRAY_SIZE(names_deltalab_gammalab_sampleaxis) == n_resolutions);
-                names = names_deltalab_gammalab_sampleaxis;
+                PROJECTION(deltalab, gammalab, sampleaxis);
                 break;
         }
         case HKL_BINOCULARS_QCUSTOM_SUB_PROJECTION_X_Y_Z:
         {
-                static const char *names_x_y_z[] = {"x", "y", "z"};
-                assert(ARRAY_SIZE(names_x_y_z) == darray_size(space->axes));
-                assert(ARRAY_SIZE(names_x_y_z) == n_resolutions);
-                names = names_x_y_z;
+                PROJECTION(x, y, z);
                 break;
         }
         case HKL_BINOCULARS_QCUSTOM_SUB_PROJECTION_Y_Z_TIMESTAMP:
         {
-                static const char *names_y_z_timestamp[] = {"y", "z", "timestamp"};
-                assert(ARRAY_SIZE(names_y_z_timestamp) == darray_size(space->axes));
-                assert(ARRAY_SIZE(names_y_z_timestamp) == n_resolutions);
-                names = names_y_z_timestamp;
+                PROJECTION(y, z, timestamp);
                 break;
         }
         case HKL_BINOCULARS_QCUSTOM_SUB_PROJECTION_Q_QPAR_QPER:
         {
-                static const char *names_q_qpar_qper[] = {"q", "qpar", "qper"};
-                assert(ARRAY_SIZE(names_q_qpar_qper) == darray_size(space->axes));
-                assert(ARRAY_SIZE(names_q_qpar_qper) == n_resolutions);
-                names = names_q_qpar_qper;
+                PROJECTION(q, qpar, qper);
                 break;
         }
         case HKL_BINOCULARS_QCUSTOM_SUB_PROJECTION_QPARS_QPER_TIMESTAMP:
         {
-                static const char *names_qpars_qper_timestamp[] = {"qpars", "qper", "timestamp"};
-                assert(ARRAY_SIZE(names_qpars_qper_timestamp) == darray_size(space->axes));
-                assert(ARRAY_SIZE(names_qpars_qper_timestamp) == n_resolutions);
-                names = names_qpars_qper_timestamp;
+                PROJECTION(qpars, qper, timestamp);
                 break;
         }
         case HKL_BINOCULARS_QCUSTOM_SUB_PROJECTION_QPAR_QPER_SAMPLEAXIS:
         {
-                static const char *names_qpar_qper_sampleaxis[] = {"qpar", "qper", "sample axis"};
-                assert(ARRAY_SIZE(names_qpar_qper_sampleaxis) == darray_size(space->axes));
-                assert(ARRAY_SIZE(names_qpar_qper_sampleaxis) == n_resolutions);
-                names = names_qpar_qper_sampleaxis;
+                PROJECTION(qpar, qper, sampleaxis);
                 break;
         }
         case HKL_BINOCULARS_QCUSTOM_SUB_PROJECTION_Q_SAMPLEAXIS_TTH:
         {
-                static const char *names_q_sampleaxis_tth[] = {"q", "sample axis", "tth"};
-                assert(ARRAY_SIZE(names_q_sampleaxis_tth) == darray_size(space->axes));
-                assert(ARRAY_SIZE(names_q_sampleaxis_tth) == n_resolutions);
-                names = names_q_sampleaxis_tth;
+                PROJECTION(q, sampleaxis, tth);
                 break;
         }
         case HKL_BINOCULARS_QCUSTOM_SUB_PROJECTION_Q_SAMPLEAXIS_TIMESTAMP:
         {
-                static const char *names_q_sampleaxis_timestamp[] = {"q", "sample axis", "timestamp"};
-                assert(ARRAY_SIZE(names_q_sampleaxis_timestamp) == darray_size(space->axes));
-                assert(ARRAY_SIZE(names_q_sampleaxis_timestamp) == n_resolutions);
-                names = names_q_sampleaxis_timestamp;
+                PROJECTION(q, sampleaxis, timestamp);
                 break;
         }
         case HKL_BINOCULARS_QCUSTOM_SUB_PROJECTION_QX_QY_TIMESTAMP:
         {
-                static const char *names_[] = {"qx", "qy", "timestamp"};
-                assert(ARRAY_SIZE(names_) == darray_size(space->axes));
-                assert(ARRAY_SIZE(names_) == n_resolutions);
-                names = names_;
+                PROJECTION(qx, qy, timestamp);
                 break;
         }
         case HKL_BINOCULARS_QCUSTOM_SUB_PROJECTION_QX_QZ_TIMESTAMP:
         {
-                static const char *names_[] = {"qx", "qz", "timestamp"};
-                assert(ARRAY_SIZE(names_) == darray_size(space->axes));
-                assert(ARRAY_SIZE(names_) == n_resolutions);
-                names = names_;
+                PROJECTION(qx, qz, timestamp);
                 break;
         }
         case HKL_BINOCULARS_QCUSTOM_SUB_PROJECTION_QY_QZ_TIMESTAMP:
         {
-                static const char *names_[] = {"qy", "qz", "timestamp"};
-                assert(ARRAY_SIZE(names_) == darray_size(space->axes));
-                assert(ARRAY_SIZE(names_) == n_resolutions);
-                names = names_;
+                PROJECTION(qy, qz, timestamp);
                 break;
         }
-        default:
+        case HKL_BINOCULARS_QCUSTOM_SUB_PROJECTION_TTH_AZIMUTH:
         {
-                static const char *names_qx_qy_qz[] = {"qx", "qy", "qz"};
-                assert(ARRAY_SIZE(names_qx_qy_qz) == darray_size(space->axes));
-                assert(ARRAY_SIZE(names_qx_qy_qz) == n_resolutions);
-                names = names_qx_qy_qz;
+                PROJECTION(tth, azimuth);
+                break;
+        }
+        case HKL_BINOCULARS_QCUSTOM_SUB_PROJECTION_Q_TIMESCAN0:
+        {
+                PROJECTION(q, timescan0);
                 break;
         }
         }
+
+	if (NULL == names)
+		PROJECTION(qx, qy, qz);
+
         return names;
 }
 
@@ -573,6 +646,62 @@ static inline double polarisation(vec3s kf, double weight, int do_polarisation)
         return weight;
 }
 
+static inline double compute_q(vec3s q)
+{
+        return glms_vec3_norm(q);
+}
+
+static inline double compute_qx(vec3s q)
+{
+        return q.raw[0];
+}
+
+static inline double compute_qy(vec3s q)
+{
+        return q.raw[1];
+}
+
+static inline double compute_qz(vec3s q)
+{
+        return q.raw[2];
+}
+
+static inline double compute_qpar(vec3s q)
+{
+        return sqrt(q.raw[0] * q.raw[0] + q.raw[1] * q.raw[1]);
+}
+
+static inline double compute_qpar_signed(vec3s q)
+{
+        float qpar = compute_qpar(q);
+
+        if(q.raw[1] != 0.0){
+                if(signbit(q.raw[1]) != 0){
+                        qpar = -qpar;
+                }
+        }else{
+                if(signbit(q.raw[0]) != 0){
+                        qpar = -qpar;
+                }
+        }
+        return qpar;
+}
+
+static inline double compute_qper(vec3s q)
+{
+        return compute_qz(q);
+}
+
+static inline double compute_tth(float q, float k)
+{
+        return asin(q / 2 / k) * 2 / M_PI * 180;
+}
+
+static inline double compute_azimuth(vec3s kf)
+{
+        return (atan2(kf.raw[2], kf.raw[1])) / M_PI * 180;
+}
+
 #define HKL_BINOCULARS_SPACE_QCUSTOM_IMPL(image_t)			\
         HKL_BINOCULARS_SPACE_QCUSTOM_DECL(image_t)			\
         {                                                               \
@@ -657,8 +786,10 @@ static inline double polarisation(vec3s kf, double weight, int do_polarisation)
                                         v = glms_vec3_sub(v , ki);      \
                                         v = glms_mat4_mulv3(m_holder_s, v, 0); \
                                                                         \
-					item.indexes_0[0] = rint(glms_vec3_norm(v) / resolutions[0]); \
-					item.indexes_0[1] = rint(asin(glms_vec3_norm(v) / 2 / k) * 2 / M_PI * 180 / resolutions[1]); \
+                                        float q = compute_q(v);         \
+                                        float tth = compute_tth(q, k);  \
+					item.indexes_0[0] = rint(q / resolutions[0]); \
+					item.indexes_0[1] = rint(tth / resolutions[1]); \
 					item.indexes_0[2] = rint(timestamp / resolutions[2]); \
                                         item.intensity = rint((double)image[i] * correction); \
                                                                         \
@@ -680,7 +811,8 @@ static inline double polarisation(vec3s kf, double weight, int do_polarisation)
                                         v = glms_vec3_sub(v , ki);      \
                                         v = glms_mat4_mulv3(m_holder_s, v, 0); \
                                                                         \
-					item.indexes_0[0] = rint(glms_vec3_norm(v) / resolutions[0]); \
+                                        float q = compute_q(v);         \
+					item.indexes_0[0] = rint(q / resolutions[0]); \
 					item.indexes_0[1] = rint(timestamp / resolutions[1]); \
 					item.indexes_0[2] = REMOVED;	\
                                         item.intensity = rint((double)image[i] * correction); \
@@ -703,8 +835,10 @@ static inline double polarisation(vec3s kf, double weight, int do_polarisation)
                                         v = glms_vec3_sub(v , ki);      \
                                         v = glms_mat4_mulv3(m_holder_s, v, 0); \
                                                                         \
-					item.indexes_0[0] = rint(sqrt(v.raw[0] * v.raw[0] + v.raw[1] * v.raw[1]) / resolutions[0]); \
-                                        item.indexes_0[1] = rint(v.raw[2] / resolutions[1]); \
+                                        float qpar = compute_qpar(v);   \
+                                        float qper = compute_qper(v);   \
+					item.indexes_0[0] = rint(qpar / resolutions[0]); \
+                                        item.indexes_0[1] = rint(qper / resolutions[1]); \
 					item.indexes_0[2] = rint(timestamp / resolutions[2]); \
                                         item.intensity = rint((double)image[i] * correction); \
                                                                         \
@@ -726,8 +860,10 @@ static inline double polarisation(vec3s kf, double weight, int do_polarisation)
                                         v = glms_vec3_sub(v , ki);      \
                                         v = glms_mat4_mulv3(m_holder_s, v, 0); \
                                                                         \
-					item.indexes_0[0] = rint(sqrt(v.raw[0] * v.raw[0] + v.raw[1] * v.raw[1]) / resolutions[0]); \
-					item.indexes_0[1] = rint(v.raw[2] / resolutions[1]); \
+                                        float qpar = compute_qpar(v);   \
+                                        float qper = compute_qper(v);   \
+					item.indexes_0[0] = rint(qpar / resolutions[0]); \
+                                        item.indexes_0[1] = rint(qper / resolutions[1]); \
 					item.indexes_0[2] = REMOVED;	\
                                         item.intensity = rint((double)image[i] * correction); \
                                                                         \
@@ -749,7 +885,8 @@ static inline double polarisation(vec3s kf, double weight, int do_polarisation)
                                         v = glms_vec3_sub(v , ki);      \
                                         v = glms_mat4_mulv3(m_holder_s, v, 0); \
                                                                         \
-					item.indexes_0[0] = rint(glms_vec3_norm(v) / resolutions[0]); \
+                                        float q = compute_q(v);         \
+					item.indexes_0[0] = rint(q / resolutions[0]); \
 					item.indexes_0[1] = rint((atan2(v.raw[2], -v.raw[1])) / M_PI * 180 / resolutions[1]); \
 					item.indexes_0[2] = rint(v.raw[0] / resolutions[2]); \
                                         item.intensity = rint((double)image[i] * correction); \
@@ -772,7 +909,8 @@ static inline double polarisation(vec3s kf, double weight, int do_polarisation)
                                         v = glms_vec3_sub(v , ki);      \
                                         v = glms_mat4_mulv3(m_holder_s, v, 0); \
                                                                         \
-					item.indexes_0[0] = rint(glms_vec3_norm(v) / resolutions[0]); \
+                                        float q = compute_q(v);         \
+					item.indexes_0[0] = rint(q / resolutions[0]); \
 					item.indexes_0[1] = rint((atan2(v.raw[2], v.raw[0])) / M_PI * 180 / resolutions[1]); \
 					item.indexes_0[2] = rint(v.raw[1] / resolutions[2]); \
                                         item.intensity = rint((double)image[i] * correction); \
@@ -795,7 +933,8 @@ static inline double polarisation(vec3s kf, double weight, int do_polarisation)
                                         v = glms_vec3_sub(v , ki);      \
                                         v = glms_mat4_mulv3(m_holder_s, v, 0); \
                                                                         \
-					item.indexes_0[0] = rint(glms_vec3_norm(v) / resolutions[0]); \
+                                        float q = compute_q(v);         \
+					item.indexes_0[0] = rint(q / resolutions[0]); \
 					item.indexes_0[1] = rint((atan2(v.raw[0], v.raw[1])) / M_PI * 180 / resolutions[1]); \
 					item.indexes_0[2] = rint(v.raw[2] / resolutions[2]); \
                                         item.intensity = rint((double)image[i] * correction); \
@@ -818,7 +957,8 @@ static inline double polarisation(vec3s kf, double weight, int do_polarisation)
                                         v = glms_vec3_sub(v , ki);      \
                                         v = glms_mat4_mulv3(m_holder_s, v, 0); \
                                                                         \
-					item.indexes_0[0] = rint(glms_vec3_norm(v) / resolutions[0]); \
+                                        float q = compute_q(v);         \
+					item.indexes_0[0] = rint(q / resolutions[0]); \
 					double ratio = v.raw[2] + item.indexes_0[0]; \
 					item.indexes_0[1] = rint(v.raw[0] / ratio / resolutions[1]); \
 					item.indexes_0[2] = rint(v.raw[1] / ratio / resolutions[2]); \
@@ -910,9 +1050,12 @@ static inline double polarisation(vec3s kf, double weight, int do_polarisation)
                                         v = glms_vec3_sub(v , ki);      \
                                         v = glms_mat4_mulv3(m_holder_s, v, 0); \
                                                                         \
-					item.indexes_0[0] = rint(glms_vec3_norm(v) / resolutions[0]); \
-					item.indexes_0[1] = rint(sqrt(v.raw[0] * v.raw[0] + v.raw[1] * v.raw[1]) / resolutions[1]); \
-					item.indexes_0[2] = rint(v.raw[2] / resolutions[2]); \
+                                        float q = compute_q(v);         \
+                                        float qpar = compute_qpar(v);	\
+                                        float qper = compute_qper(v);   \
+					item.indexes_0[0] = rint(q / resolutions[0]); \
+					item.indexes_0[1] = rint(qpar / resolutions[1]); \
+					item.indexes_0[2] = rint(qper / resolutions[2]); \
                                         item.intensity = rint((double)image[i] * correction); \
                                                                         \
                                         if(TRUE == item_in_the_limits(&item, limits, n_limits)) \
@@ -933,20 +1076,13 @@ static inline double polarisation(vec3s kf, double weight, int do_polarisation)
                                         v = glms_vec3_sub(v , ki);      \
                                         v = glms_mat4_mulv3(m_holder_s, v, 0); \
                                                                         \
-					item.indexes_0[0] = rint(sqrt(v.raw[0] * v.raw[0] + v.raw[1] * v.raw[1]) / resolutions[0]); \
-                                        item.indexes_0[1] = rint(v.raw[2] / resolutions[1]); \
+                                        float qpars = compute_qpar_signed(v); \
+                                        float qper = compute_qper(v);   \
+					item.indexes_0[0] = rint(qpars / resolutions[0]); \
+                                        item.indexes_0[1] = rint(qper / resolutions[1]); \
 					item.indexes_0[2] = rint(timestamp / resolutions[2]); \
                                         item.intensity = rint((double)image[i] * correction); \
                                                                         \
-                                        if(v.raw[1] != 0.0){            \
-                                                if(signbit(v.raw[1]) != 0){ \
-                                                        item.indexes_0[0] = -item.indexes_0[0]; \
-                                                }                       \
-                                        }else{                          \
-                                                if(signbit(v.raw[0]) != 0){ \
-                                                        item.indexes_0[0] = -item.indexes_0[0]; \
-                                                }                       \
-                                        }                               \
                                         if(TRUE == item_in_the_limits(&item, limits, n_limits)) \
                                                 darray_append(space->items, item); \
                                 }                                       \
@@ -969,8 +1105,10 @@ static inline double polarisation(vec3s kf, double weight, int do_polarisation)
                                                 v = glms_vec3_sub(v , ki); \
                                                 v = glms_mat4_mulv3(m_holder_s, v, 0); \
                                                                         \
-                                                item.indexes_0[0] = rint(sqrt(v.raw[0] * v.raw[0] + v.raw[1] * v.raw[1]) / resolutions[0]); \
-                                                item.indexes_0[1] = rint(v.raw[2] / resolutions[1]); \
+                                                float qpar = compute_qpar(v); \
+                                                float qper = compute_qper(v); \
+                                                item.indexes_0[0] = rint(qpar / resolutions[0]); \
+                                                item.indexes_0[1] = rint(qper / resolutions[1]); \
                                                 item.indexes_0[2] = axis; \
                                                 item.intensity = rint((double)image[i] * correction); \
                                                                         \
@@ -997,9 +1135,11 @@ static inline double polarisation(vec3s kf, double weight, int do_polarisation)
                                                 v = glms_vec3_sub(v , ki); \
                                                 v = glms_mat4_mulv3(m_holder_s, v, 0); \
                                                                         \
-                                                item.indexes_0[0] = rint(glms_vec3_norm(v) / resolutions[0]); \
+                                                float q = compute_q(v); \
+                                                double tth = compute_tth(q, k); \
+                                                item.indexes_0[0] = rint(q / resolutions[0]); \
                                                 item.indexes_0[1] = axis; \
-                                                item.indexes_0[2] = rint(asin(glms_vec3_norm(v) / 2 / k) * 2 / M_PI * 180 / resolutions[2]); \
+                                                item.indexes_0[2] = rint(tth / resolutions[2]); \
                                                 item.intensity = rint((double)image[i] * correction); \
                                                                         \
                                                 if(TRUE == item_in_the_limits(&item, limits, n_limits)) \
@@ -1025,7 +1165,8 @@ static inline double polarisation(vec3s kf, double weight, int do_polarisation)
                                                 v = glms_vec3_sub(v , ki); \
                                                 v = glms_mat4_mulv3(m_holder_s, v, 0); \
                                                                         \
-                                                item.indexes_0[0] = rint(glms_vec3_norm(v) / resolutions[0]); \
+                                                float q = compute_q(v); \
+                                                item.indexes_0[0] = rint(q / resolutions[0]); \
                                                 item.indexes_0[1] = axis; \
                                                 item.indexes_0[2] = rint(timestamp / resolutions[2]); \
                                                 item.intensity = rint((double)image[i] * correction); \
@@ -1106,6 +1247,56 @@ static inline double polarisation(vec3s kf, double weight, int do_polarisation)
                         }                                               \
                         break;                                          \
                 }                                                       \
+                case HKL_BINOCULARS_QCUSTOM_SUB_PROJECTION_TTH_AZIMUTH: \
+                {                                                       \
+                        for(i=0;i<n_pixels;++i){                        \
+                                if(not_masked(masked, i)){              \
+                                        CGLM_ALIGN_MAT vec3s v = {{q_x[i], q_y[i], q_z[i]}}; \
+                                                                        \
+                                        v = glms_mat4_mulv3(m_holder_d, v, 1); \
+                                        double azimuth = compute_azimuth(v); \
+                                        v = glms_vec3_scale_as(v, k);   \
+                                        correction = polarisation(v, weight, do_polarisation_correction); \
+                                        v = glms_vec3_sub(v , ki);      \
+                                        v = glms_mat4_mulv3(m_holder_s, v, 0); \
+                                                                        \
+                                        float q = compute_q(v);         \
+                                        double tth = compute_tth(q, k); \
+					item.indexes_0[0] = rint(tth / resolutions[0]); \
+					item.indexes_0[1] = rint(azimuth / resolutions[1]); \
+					item.indexes_0[2] = REMOVED;    \
+                                        item.intensity = rint((double)image[i] * correction); \
+                                                                        \
+                                        if(TRUE == item_in_the_limits(&item, limits, n_limits)) \
+                                                darray_append(space->items, item); \
+                                }                                       \
+                        }                                               \
+                        break;                                          \
+                }                                                       \
+                case HKL_BINOCULARS_QCUSTOM_SUB_PROJECTION_Q_TIMESCAN0:	\
+                {                                                       \
+                        for(i=0;i<n_pixels;++i){                        \
+                                if(not_masked(masked, i)){              \
+                                        CGLM_ALIGN_MAT vec3s v = {{q_x[i], q_y[i], q_z[i]}}; \
+                                                                        \
+                                        v = glms_mat4_mulv3(m_holder_d, v, 1); \
+                                        v = glms_vec3_scale_as(v, k);   \
+                                        correction = polarisation(v, weight, do_polarisation_correction); \
+                                        v = glms_vec3_sub(v , ki);      \
+                                        v = glms_mat4_mulv3(m_holder_s, v, 0); \
+                                                                        \
+                                        float q = compute_q(v);         \
+					item.indexes_0[0] = rint(q / resolutions[0]); \
+					item.indexes_0[1] = rint(timescan0 / resolutions[1]); \
+					item.indexes_0[2] = REMOVED;	\
+                                        item.intensity = rint((double)image[i] * correction); \
+                                                                        \
+                                        if(TRUE == item_in_the_limits(&item, limits, n_limits)) \
+                                                darray_append(space->items, item); \
+                                }                                       \
+                        }                                               \
+                        break;                                          \
+                }                                                       \
                 }                                                       \
 		space_update_axes(space, names, n_pixels, resolutions);	\
 		hkl_detector_free(detector);				\
