@@ -57,9 +57,8 @@ import           Data.Text.Encoding                (decodeUtf8, encodeUtf8)
 import           Data.Text.IO                      (putStr)
 import           Data.Vector.Storable.Mutable      (unsafeWith)
 import           Foreign.C.Types                   (CDouble (..))
-import           Foreign.ForeignPtr                (ForeignPtr, withForeignPtr)
+import           Foreign.ForeignPtr                (withForeignPtr)
 import           GHC.Generics                      (Generic)
-import           Generic.Random                    (genericArbitraryU)
 import           Numeric.Units.Dimensional.Prelude (Angle, degree, radian, (*~),
                                                     (/~))
 import           Path                              (Abs, Dir, Path)
@@ -67,7 +66,6 @@ import           Pipes                             (await, each, runEffect,
                                                     yield, (>->))
 import           Pipes.Prelude                     (filter, map, tee, toListM)
 import           Pipes.Safe                        (runSafeT)
-import           Test.QuickCheck                   (Arbitrary (..))
 import           Text.Printf                       (printf)
 
 import           Hkl.Binoculars.Common
@@ -76,7 +74,6 @@ import           Hkl.Binoculars.Config.Common
 import           Hkl.Binoculars.Pipes
 import           Hkl.Binoculars.Projections
 import           Hkl.C.Binoculars
-import           Hkl.C.Hkl
 import           Hkl.DataSource
 import           Hkl.Detector
 import           Hkl.Geometry
@@ -97,9 +94,10 @@ import           Hkl.Utils
 data DataFrameQCustom
     = DataFrameQCustom
       Attenuation -- attenuation
-      (ForeignPtr C'HklGeometry) -- geometry
+      Geometry -- geometry
       Image -- image
       Timestamp -- timestamp in double
+      Timescan0 -- timescan0 in double
     deriving Show
 
 instance DataSource DataFrameQCustom where
@@ -109,6 +107,7 @@ instance DataSource DataFrameQCustom where
       (DataSourcePath Geometry)
       (DataSourcePath Image)
       (DataSourcePath Timestamp)
+      (DataSourcePath Timescan0)
     deriving (Generic, Show, FromJSON, ToJSON)
 
   data DataSourceAcq DataFrameQCustom
@@ -117,23 +116,23 @@ instance DataSource DataFrameQCustom where
       (DataSourceAcq Geometry)
       (DataSourceAcq Image)
       (DataSourceAcq Timestamp)
+      (DataSourceAcq Timescan0)
 
-  withDataSourceP f (DataSourcePath'DataFrameQCustom a g i idx) gg =
+  withDataSourceP f (DataSourcePath'DataFrameQCustom a g i idx t0) gg =
     withDataSourceP f a $ \a' ->
     withDataSourceP f g $ \g' ->
     withDataSourceP f i $ \i' ->
-    withDataSourceP f idx $ \idx' -> gg (DataSourceAcq'DataFrameQCustom a' g' i' idx')
+    withDataSourceP f idx $ \idx' ->
+    withDataSourceP f t0 $ \t0' -> gg (DataSourceAcq'DataFrameQCustom a' g' i' idx' t0')
 
 instance Is1DStreamable (DataSourceAcq DataFrameQCustom) DataFrameQCustom where
-    extract1DStreamValue (DataSourceAcq'DataFrameQCustom att geom img idx) i =
+    extract1DStreamValue (DataSourceAcq'DataFrameQCustom att geom img idx t0) i =
       DataFrameQCustom
       <$> extract1DStreamValue att i
       <*> extract1DStreamValue geom i
       <*> extract1DStreamValue img i
       <*> extract1DStreamValue idx i
-
-instance Arbitrary (DataSourcePath DataFrameQCustom) where
-  arbitrary = DataSourcePath'DataFrameQCustom <$> arbitrary <*> arbitrary <*> arbitrary <*> arbitrary
+      <*> extract0DStreamValue t0
 
 default'DataSourcePath'DataFrameQCustom :: DataSourcePath DataFrameQCustom
 default'DataSourcePath'DataFrameQCustom
@@ -142,17 +141,21 @@ default'DataSourcePath'DataFrameQCustom
       (DataSourcePath'Float (hdf5p $ grouppat 0 $ datasetp "scan_data/attenuation"))
       2 0 Nothing)
     (DataSourcePath'Geometry
-      (Geometry'Factory Uhv)
+      (Geometry'Factory Uhv Nothing)
       (DataSourcePath'Double (hdf5p $ grouppat 0 $ datasetp "SIXS/Monochromator/wavelength"))
-      [ DataSourcePath'Double(hdf5p $ grouppat 0 $ datasetp "scan_data/UHV_MU")
-      , DataSourcePath'Double(hdf5p $ grouppat 0 $ datasetp "scan_data/UHV_OMEGA")
-      , DataSourcePath'Double(hdf5p $ grouppat 0 $ datasetp "scan_data/UHV_DELTA")
-      , DataSourcePath'Double(hdf5p $ grouppat 0 $ datasetp "scan_data/UHV_GAMMA")
-      ])
+      (DataSourcePath'List
+       [ DataSourcePath'Double(hdf5p $ grouppat 0 $ datasetp "scan_data/UHV_MU")
+       , DataSourcePath'Double(hdf5p $ grouppat 0 $ datasetp "scan_data/UHV_OMEGA")
+       , DataSourcePath'Double(hdf5p $ grouppat 0 $ datasetp "scan_data/UHV_DELTA")
+       , DataSourcePath'Double(hdf5p $ grouppat 0 $ datasetp "scan_data/UHV_GAMMA")
+       ]
+      )
+    )
     (DataSourcePath'Image
       (hdf5p $ grouppat 0 $ datasetp "scan_data/xpad_image")
       defaultDetector)
     (DataSourcePath'Timestamp(hdf5p $ grouppat 0 $ datasetp "scan_data/epoch"))
+    (DataSourcePath'Timescan0(hdf5p $ grouppat 0 $ datasetp "scan_data/epoch"))
 
 instance HasFieldComment (DataSourcePath DataFrameQCustom) where
   fieldComment _ = [ "`datapath` internal value used to find the data in the data file."
@@ -187,9 +190,6 @@ data instance Config 'QCustomProjection
     , binocularsConfig'QCustom'SampleAxis             :: Maybe SampleAxis
     } deriving (Show, Generic)
 
-instance Arbitrary (Config 'QCustomProjection) where
-  arbitrary = genericArbitraryU
-
 newtype instance Args 'QCustomProjection = Args'QCustomProjection (Maybe ConfigRange)
 
 default'BinocularsConfig'QCustom :: Config 'QCustomProjection
@@ -251,6 +251,8 @@ instance HasIniConfig 'QCustomProjection where
                                                 HklBinocularsQCustomSubProjectionEnum'QxQyTimestamp -> Nothing
                                                 HklBinocularsQCustomSubProjectionEnum'QxQzTimestamp -> Nothing
                                                 HklBinocularsQCustomSubProjectionEnum'QyQzTimestamp -> Nothing
+                                                HklBinocularsQCustomSubProjectionEnum'TthAzimuth -> Nothing
+                                                HklBinocularsQCustomSubProjectionEnum'QTimescan0 -> Nothing
 
 
     BinocularsConfig'QCustom
@@ -430,6 +432,36 @@ overloadTimestampPath msub idx =
                    HklBinocularsQCustomSubProjectionEnum'QxQyTimestamp -> idx
                    HklBinocularsQCustomSubProjectionEnum'QxQzTimestamp -> idx
                    HklBinocularsQCustomSubProjectionEnum'QyQzTimestamp -> idx
+                   HklBinocularsQCustomSubProjectionEnum'TthAzimuth -> DataSourcePath'Timestamp'NoTimestamp
+                   HklBinocularsQCustomSubProjectionEnum'QTimescan0 -> DataSourcePath'Timestamp'NoTimestamp
+
+overloadTimescan0Path :: Maybe HklBinocularsQCustomSubProjectionEnum -> DataSourcePath Timescan0 -> DataSourcePath Timescan0
+overloadTimescan0Path msub idx =
+  case msub of
+    Nothing -> DataSourcePath'Timescan0'NoTimescan0
+    (Just sub) -> case sub of
+                   HklBinocularsQCustomSubProjectionEnum'QxQyQz -> DataSourcePath'Timescan0'NoTimescan0
+                   HklBinocularsQCustomSubProjectionEnum'QTthTimestamp -> DataSourcePath'Timescan0'NoTimescan0
+                   HklBinocularsQCustomSubProjectionEnum'QTimestamp -> DataSourcePath'Timescan0'NoTimescan0
+                   HklBinocularsQCustomSubProjectionEnum'QparQperTimestamp -> DataSourcePath'Timescan0'NoTimescan0
+                   HklBinocularsQCustomSubProjectionEnum'QparQper -> DataSourcePath'Timescan0'NoTimescan0
+                   HklBinocularsQCustomSubProjectionEnum'QPhiQx -> DataSourcePath'Timescan0'NoTimescan0
+                   HklBinocularsQCustomSubProjectionEnum'QPhiQy -> DataSourcePath'Timescan0'NoTimescan0
+                   HklBinocularsQCustomSubProjectionEnum'QPhiQz -> DataSourcePath'Timescan0'NoTimescan0
+                   HklBinocularsQCustomSubProjectionEnum'QStereo -> DataSourcePath'Timescan0'NoTimescan0
+                   HklBinocularsQCustomSubProjectionEnum'DeltalabGammalabSampleaxis -> DataSourcePath'Timescan0'NoTimescan0
+                   HklBinocularsQCustomSubProjectionEnum'XYZ -> DataSourcePath'Timescan0'NoTimescan0
+                   HklBinocularsQCustomSubProjectionEnum'YZTimestamp -> DataSourcePath'Timescan0'NoTimescan0
+                   HklBinocularsQCustomSubProjectionEnum'QQparQper -> DataSourcePath'Timescan0'NoTimescan0
+                   HklBinocularsQCustomSubProjectionEnum'QparsQperTimestamp -> DataSourcePath'Timescan0'NoTimescan0
+                   HklBinocularsQCustomSubProjectionEnum'QparQperSampleaxis -> DataSourcePath'Timescan0'NoTimescan0
+                   HklBinocularsQCustomSubProjectionEnum'QSampleaxisTth -> DataSourcePath'Timescan0'NoTimescan0
+                   HklBinocularsQCustomSubProjectionEnum'QSampleaxisTimestamp -> DataSourcePath'Timescan0'NoTimescan0
+                   HklBinocularsQCustomSubProjectionEnum'QxQyTimestamp -> DataSourcePath'Timescan0'NoTimescan0
+                   HklBinocularsQCustomSubProjectionEnum'QxQzTimestamp -> DataSourcePath'Timescan0'NoTimescan0
+                   HklBinocularsQCustomSubProjectionEnum'QyQzTimestamp -> DataSourcePath'Timescan0'NoTimescan0
+                   HklBinocularsQCustomSubProjectionEnum'TthAzimuth -> DataSourcePath'Timescan0'NoTimescan0
+                   HklBinocularsQCustomSubProjectionEnum'QTimescan0 -> idx
 
 overloadWaveLength :: Maybe Double -> DataSourcePath Double -> DataSourcePath Double
 overloadWaveLength ma wp = maybe wp DataSourcePath'Double'Const ma
@@ -446,7 +478,7 @@ overload'DataSourcePath'DataFrameQCustom :: BinocularsConfig'Common
                                          -> Maybe HklBinocularsQCustomSubProjectionEnum
                                          -> DataSourcePath DataFrameQCustom
                                          -> DataSourcePath DataFrameQCustom
-overload'DataSourcePath'DataFrameQCustom common msub (DataSourcePath'DataFrameQCustom attenuationPath' geometryPath imagePath indexP)
+overload'DataSourcePath'DataFrameQCustom common msub (DataSourcePath'DataFrameQCustom attenuationPath' geometryPath imagePath indexP timescan0P)
   = let mAttCoef = binocularsConfig'Common'AttenuationCoefficient common
         mMaxAtt = binocularsConfig'Common'AttenuationMax common
         mWavelength = binocularsConfig'Common'Wavelength common
@@ -456,8 +488,9 @@ overload'DataSourcePath'DataFrameQCustom common msub (DataSourcePath'DataFrameQC
         newGeometryPath = overloadGeometryPath mWavelength geometryPath
         newImagePath = overloadImagePath detector imagePath
         newTimestampPath = overloadTimestampPath msub indexP
+        newTimescan0Path = overloadTimescan0Path msub timescan0P
     in
-      DataSourcePath'DataFrameQCustom newAttenuationPath newGeometryPath newImagePath newTimestampPath
+      DataSourcePath'DataFrameQCustom newAttenuationPath newGeometryPath newImagePath newTimestampPath newTimescan0Path
 
 
 guess'DataSourcePath'DataFrameQCustom :: BinocularsConfig'Common
@@ -505,6 +538,15 @@ guess'DataSourcePath'DataFrameQCustom common msub cfg =
           mkTimeStamp'Fly msub'
             = overloadTimestampPath msub' (DataSourcePath'Timestamp(hdf5p $ grouppat 0 $ datasetp "scan_data/epoch"))
 
+      -- timescan0
+      let mkTimescan0'Sbs :: Maybe HklBinocularsQCustomSubProjectionEnum -> DataSourcePath Timescan0
+          mkTimescan0'Sbs msub'
+            = overloadTimescan0Path msub' (DataSourcePath'Timescan0(hdf5p $ grouppat 0 $ datasetp "scan_data/sensors_timestamps"))
+
+      let mkTimescan0'Fly :: Maybe HklBinocularsQCustomSubProjectionEnum -> DataSourcePath Timescan0
+          mkTimescan0'Fly msub'
+            = overloadTimescan0Path msub' (DataSourcePath'Timescan0(hdf5p $ grouppat 0 $ datasetp "scan_data/epoch"))
+
       -- wavelength
       let dataSourcePath'WaveLength'Mars :: DataSourcePath Double
           dataSourcePath'WaveLength'Mars
@@ -585,16 +627,16 @@ guess'DataSourcePath'DataFrameQCustom common msub cfg =
       let dataSourcePath'Geometry'Sixs'Uhv :: DataSourcePath Geometry
           dataSourcePath'Geometry'Sixs'Uhv
             = DataSourcePath'Geometry
-              (Geometry'Factory Uhv)
+              (Geometry'Factory Uhv Nothing)
               (overloadWaveLength mWavelength dataSourcePath'WaveLength'Sixs)
-              [sixs'Uhv'Mu, sixs'Uhv'Omega, sixs'Uhv'Delta, sixs'Uhv'Gamma]
+              (DataSourcePath'List [sixs'Uhv'Mu, sixs'Uhv'Omega, sixs'Uhv'Delta, sixs'Uhv'Gamma])
 
       let dataSourcePath'Geometry'Sixs'UhvGisaxs :: DataSourcePath Geometry
           dataSourcePath'Geometry'Sixs'UhvGisaxs
             = DataSourcePath'Geometry
               sixsUhvGisaxs
               (overloadWaveLength mWavelength dataSourcePath'WaveLength'Sixs)
-              [ sixs'Uhv'Mu, sixs'Uhv'Omega, sixs'eix, sixs'eiz ]
+              (DataSourcePath'List [ sixs'Uhv'Mu, sixs'Uhv'Omega, sixs'eix, sixs'eiz ])
 
       let sixs'Med'Beta
             = DataSourcePath'Double'Ini cfg "geometry.values" "beta"
@@ -659,30 +701,30 @@ guess'DataSourcePath'DataFrameQCustom common msub cfg =
       let dataSourcePath'Geometry'Sixs'MedH ::  DataSourcePath Geometry
           dataSourcePath'Geometry'Sixs'MedH
             = DataSourcePath'Geometry
-              (Geometry'Factory MedH)
+              (Geometry'Factory MedH Nothing)
               (overloadWaveLength mWavelength dataSourcePath'WaveLength'Sixs)
-              [ sixs'Med'Beta, sixs'MedH'Mu, sixs'MedH'Gamma, sixs'MedH'Delta ]
+              (DataSourcePath'List [ sixs'Med'Beta, sixs'MedH'Mu, sixs'MedH'Gamma, sixs'MedH'Delta ])
 
       let dataSourcePath'Geometry'Sixs'MedHGisaxs ::  DataSourcePath Geometry
           dataSourcePath'Geometry'Sixs'MedHGisaxs
             = DataSourcePath'Geometry
               sixsMedHGisaxs
               (overloadWaveLength mWavelength dataSourcePath'WaveLength'Sixs)
-              [ sixs'Med'Beta, sixs'MedH'Mu, sixs'eix, sixs'eiz ]
+              (DataSourcePath'List [ sixs'Med'Beta, sixs'MedH'Mu, sixs'eix, sixs'eiz ])
 
       let dataSourcePath'Geometry'Sixs'MedV :: DataSourcePath Geometry
           dataSourcePath'Geometry'Sixs'MedV
             = DataSourcePath'Geometry
-              (Geometry'Factory MedV)
+              (Geometry'Factory MedV Nothing)
               (overloadWaveLength mWavelength dataSourcePath'WaveLength'Sixs)
-              [ sixs'Med'Beta, sixs'MedV'Mu, sixs'MedV'Omega, sixs'MedV'Gamma, sixs'MedV'Delta, sixs'MedV'Etaa ]
+              (DataSourcePath'List [ sixs'Med'Beta, sixs'MedV'Mu, sixs'MedV'Omega, sixs'MedV'Gamma, sixs'MedV'Delta, sixs'MedV'Etaa ])
 
       let dataSourcePath'Geometry'Sixs'MedVGisaxs :: DataSourcePath Geometry
           dataSourcePath'Geometry'Sixs'MedVGisaxs
             = DataSourcePath'Geometry
               sixsMedVGisaxs
               (overloadWaveLength mWavelength dataSourcePath'WaveLength'Sixs)
-              [ sixs'Med'Beta, sixs'MedV'Mu, sixs'MedV'Omega, sixs'eix, sixs'eiz ]
+              (DataSourcePath'List [ sixs'Med'Beta, sixs'MedV'Mu, sixs'MedV'Omega, sixs'eix, sixs'eiz ])
 
       let dataSourcePath'DataFrameQCustom'Sixs'Fly :: DataSourcePath Geometry -> DataSourcePath DataFrameQCustom
           dataSourcePath'DataFrameQCustom'Sixs'Fly g
@@ -691,6 +733,7 @@ guess'DataSourcePath'DataFrameQCustom common msub cfg =
               g
               (mkDetector'Sixs'Fly detector)
               (mkTimeStamp'Fly msub)
+              (mkTimescan0'Fly msub)
 
       let dataSourcePath'DataFrameQCustom'Sixs'Sbs :: DataSourcePath Geometry -> DataSourcePath DataFrameQCustom
           dataSourcePath'DataFrameQCustom'Sixs'Sbs g
@@ -699,76 +742,90 @@ guess'DataSourcePath'DataFrameQCustom common msub cfg =
               g
               (mkDetector'Sixs'Sbs detector)
               (mkTimeStamp'Sbs msub)
+              (mkTimescan0'Sbs msub)
 
       case inputtype of
          CristalK6C -> DataSourcePath'DataFrameQCustom
                       (mkAttenuation mAttenuationCoefficient  DataSourcePath'NoAttenuation)
                       (DataSourcePath'Geometry
-                        (Geometry'Factory K6c)
+                        (Geometry'Factory K6c Nothing)
                         (overloadWaveLength mWavelength (DataSourcePath'Double (hdf5p $ grouppat 0 $ datasetp "CRISTAL/Monochromator/lambda")))
-                        [ DataSourcePath'Double (hdf5p $ grouppat 0 $ datasetp "CRISTAL/Diffractometer/i06-c-c07-ex-dif-mu/position")
-                        , DataSourcePath'Double (hdf5p $ grouppat 0 $ datasetp "CRISTAL/Diffractometer/i06-c-c07-ex-dif-komega/position")
-                        , DataSourcePath'Double (hdf5p $ grouppat 0 $ datasetp "CRISTAL/Diffractometer/i06-c-c07-ex-dif-kappa/position")
-                        , DataSourcePath'Double (hdf5p $ grouppat 0 $ datasetp "scan_data/actuator_1_1")
-                        , DataSourcePath'Double (hdf5p $ grouppat 0 $ datasetp "CRISTAL/Diffractometer/i06-c-c07-ex-dif-gamma/position")
-                        , DataSourcePath'Double (hdf5p $ grouppat 0 $ datasetp "CRISTAL/Diffractometer/i06-c-c07-ex-dif-delta/position")
-                        ]
+                        (DataSourcePath'List
+                         [ DataSourcePath'Double (hdf5p $ grouppat 0 $ datasetp "CRISTAL/Diffractometer/i06-c-c07-ex-dif-mu/position")
+                         , DataSourcePath'Double (hdf5p $ grouppat 0 $ datasetp "CRISTAL/Diffractometer/i06-c-c07-ex-dif-komega/position")
+                         , DataSourcePath'Double (hdf5p $ grouppat 0 $ datasetp "CRISTAL/Diffractometer/i06-c-c07-ex-dif-kappa/position")
+                         , DataSourcePath'Double (hdf5p $ grouppat 0 $ datasetp "scan_data/actuator_1_1")
+                         , DataSourcePath'Double (hdf5p $ grouppat 0 $ datasetp "CRISTAL/Diffractometer/i06-c-c07-ex-dif-gamma/position")
+                         , DataSourcePath'Double (hdf5p $ grouppat 0 $ datasetp "CRISTAL/Diffractometer/i06-c-c07-ex-dif-delta/position")
+                         ]
+                        )
                       )
                       (DataSourcePath'Image
                         (hdf5p $ grouppat 0 $ datasetp "scan_data/data_05")
                         detector)
                       (mkTimeStamp'Sbs msub)
+                      (mkTimescan0'Sbs msub)
          MarsFlyscan -> DataSourcePath'DataFrameQCustom
                        (mkAttenuation mAttenuationCoefficient DataSourcePath'NoAttenuation)
                        -- (mkAttenuation mAttenuationCoefficient (DataSourcePath'ApplyedAttenuationFactor
                        --                                         (DataSourcePath'Float (hdf5p $ grouppat 0 $ datasetp "scan_data/applied_att"))))
                        (DataSourcePath'Geometry
-                         (Geometry'Factory Mars)
+                         (Geometry'Factory Mars Nothing)
                          (overloadWaveLength mWavelength dataSourcePath'WaveLength'Mars)
-                         [ DataSourcePath'Double(hdf5p $ grouppat 0 $ datasetp "scan_data/omega")
-                         , DataSourcePath'Double(hdf5p $ grouppat 0 $ datasetp "scan_data/chi")
-                         , DataSourcePath'Double(hdf5p $ grouppat 0 $ datasetp "scan_data/phi")
-                         , DataSourcePath'Double(hdf5p $ grouppat 0 $ datasetp "scan_data/tth")
-                         ])
+                         (DataSourcePath'List
+                          [ DataSourcePath'Double(hdf5p $ grouppat 0 $ datasetp "scan_data/omega")
+                          , DataSourcePath'Double(hdf5p $ grouppat 0 $ datasetp "scan_data/chi")
+                          , DataSourcePath'Double(hdf5p $ grouppat 0 $ datasetp "scan_data/phi")
+                          , DataSourcePath'Double(hdf5p $ grouppat 0 $ datasetp "scan_data/tth")
+                          ]
+                         )
+                        )
                        (DataSourcePath'Image
                          (hdf5p $ grouppat 0 (datasetp "scan_data/merlin_image"
                                               `H5Or`
                                               datasetp "scan_data/merlin_quad_image"))
                          detector)
                        (mkTimeStamp'Fly msub)
+                       (mkTimescan0'Sbs msub)
          MarsSbs -> DataSourcePath'DataFrameQCustom
                    (mkAttenuation mAttenuationCoefficient DataSourcePath'NoAttenuation)
                    (DataSourcePath'Geometry
-                     (Geometry'Factory Mars)
+                     (Geometry'Factory Mars Nothing)
                      (overloadWaveLength mWavelength dataSourcePath'WaveLength'Mars)
-                     [ DataSourcePath'Double(hdf5p $ grouppat 0 $ (datasetp "scan_data/omega"
-                                                                   `H5Or`
-                                                                   datasetp "MARS/d03-1-cx2__ex__dif-mt_rx.1_#2/raw_value"))
+                     (DataSourcePath'List
+                      [ DataSourcePath'Double(hdf5p $ grouppat 0 $ (datasetp "scan_data/omega"
+                                                                    `H5Or`
+                                                                     datasetp "MARS/d03-1-cx2__ex__dif-mt_rx.1_#2/raw_value"))
 
-                       `DataSourcePath'Double'Or`
-                       DataSourcePath'Double'Const 0
-                     , DataSourcePath'Double(hdf5p $ grouppat 0 $ (datasetp "scan_data/chi"
-                                                                   `H5Or`
-                                                                   datasetp "MARS/d03-1-cx2__ex__gonio-mt_rs_#2/raw_value"))
-                       `DataSourcePath'Double'Or`
-                       DataSourcePath'Double'Const 0
-                     , DataSourcePath'Double(hdf5p $ grouppat 0 $ (datasetp "scan_data/phi"
-                                                                   `H5Or`
-                                                                   datasetpattr ("long_name", "d03-1-cx2/ex/gonio-mt_rz/position")
-                                                                   `H5Or`
-                                                                   datasetp "MARS/d03-1-cx2__ex__gonio-mt_rz_#2/raw_value"))
-                       `DataSourcePath'Double'Or`
-                       DataSourcePath'Double'Const 0
-                     , DataSourcePath'Double(hdf5p $ grouppat 0 $ (datasetp "scan_data/tth"
-                                                                   `H5Or`
-                                                                   datasetp "MARS/d03-1-cx2__ex__dif-mt_rx.2_#2/raw_value"))
-                       `DataSourcePath'Double'Or`
-                       DataSourcePath'Double'Const 0
-                     ])
+                        `DataSourcePath'Double'Or`
+                         DataSourcePath'Double'Const 0
+                      , DataSourcePath'Double(hdf5p $ grouppat 0 $ (datasetp "scan_data/chi"
+                                                                    `H5Or`
+                                                                    datasetp "MARS/d03-1-cx2__ex__gonio-mt_rs_#2/raw_value"))
+                        `DataSourcePath'Double'Or`
+                        DataSourcePath'Double'Const 0
+                      , DataSourcePath'Double(hdf5p $ grouppat 0 $ (datasetp "scan_data/phi"
+                                                                    `H5Or`
+                                                                    datasetpattr ("long_name", "d03-1-cx2/ex/gonio-mt_rz/position")
+                                                                    `H5Or`
+                                                                    datasetp "MARS/d03-1-cx2__ex__gonio-mt_rz_#2/raw_value"))
+                        `DataSourcePath'Double'Or`
+                        DataSourcePath'Double'Const 0
+                      , DataSourcePath'Double(hdf5p $ grouppat 0 $ (datasetp "scan_data/tth"
+                                                                    `H5Or`
+                                                                    datasetp "MARS/d03-1-cx2__ex__dif-mt_rx.2_#2/raw_value"))
+                        `DataSourcePath'Double'Or`
+                        DataSourcePath'Double'Const 0
+                      ]
+                     )
+                    )
                    (DataSourcePath'Image
-                    (hdf5p $ datasetpattr ("long_name", "d03-1-c00/dt/merlin-quad/image"))
+                    (hdf5p $ (datasetpattr ("long_name", "d03-1-c00/dt/merlin-quad/image")
+                             `H5Or`
+                             datasetpattr ("interpretation", "image")))
                     detector)
                    (mkTimeStamp'Sbs msub)
+                   (mkTimescan0'Sbs msub)
          SixsFlyMedH -> dataSourcePath'DataFrameQCustom'Sixs'Fly dataSourcePath'Geometry'Sixs'MedH
          SixsFlyMedHGisaxs -> dataSourcePath'DataFrameQCustom'Sixs'Fly dataSourcePath'Geometry'Sixs'MedHGisaxs
          SixsFlyMedV -> dataSourcePath'DataFrameQCustom'Sixs'Fly dataSourcePath'Geometry'Sixs'MedV
@@ -797,9 +854,9 @@ spaceQCustom :: Detector a DIM2
              -> Space DIM3
              -> DataFrameQCustom
              -> IO (DataFrameSpace DIM3)
-spaceQCustom det pixels rs mmask' surf mlimits subprojection uqx uqy uqz mSampleAxis doPolarizationCorrection space@(Space fSpace) (DataFrameQCustom att g img index) =
+spaceQCustom det pixels rs mmask' surf mlimits subprojection uqx uqy uqz mSampleAxis doPolarizationCorrection space@(Space fSpace) (DataFrameQCustom att g img index timescan0) =
   withNPixels det $ \nPixels ->
-  withForeignPtr g $ \geometry ->
+  withGeometry g $ \geometry ->
   withForeignPtr (toForeignPtr pixels) $ \pix ->
   withResolutions rs $ \nr r ->
   withPixelsDims pixels $ \ndim dims ->
@@ -809,11 +866,11 @@ spaceQCustom det pixels rs mmask' surf mlimits subprojection uqx uqy uqz mSample
   withForeignPtr fSpace $ \pSpace -> do
   case img of
     (ImageInt32 arr) -> unsafeWith arr $ \i -> do
-      {-# SCC "hkl_binoculars_space_qcustom_int32_t" #-} c'hkl_binoculars_space_qcustom_int32_t pSpace geometry i nPixels (CDouble . unAttenuation $ att) pix (toEnum ndim) dims r (toEnum nr) mask'' (toEnum $ fromEnum surf) limits (toEnum nlimits) (CDouble . unTimestamp $ index) (toEnum . fromEnum $ subprojection) (CDouble (uqx /~ radian)) (CDouble (uqy /~ radian)) (CDouble (uqz /~ radian)) sampleAxis (toEnum . fromEnum $ doPolarizationCorrection)
+      {-# SCC "hkl_binoculars_space_qcustom_int32_t" #-} c'hkl_binoculars_space_qcustom_int32_t pSpace geometry i nPixels (CDouble . unAttenuation $ att) pix (toEnum ndim) dims r (toEnum nr) mask'' (toEnum $ fromEnum surf) limits (toEnum nlimits) (CDouble . unTimestamp $ index) (CDouble . unTimescan0 $ timescan0) (toEnum . fromEnum $ subprojection) (CDouble (uqx /~ radian)) (CDouble (uqy /~ radian)) (CDouble (uqz /~ radian)) sampleAxis (toEnum . fromEnum $ doPolarizationCorrection)
     (ImageWord16 arr) -> unsafeWith arr $ \i -> do
-      {-# SCC "hkl_binoculars_space_qcustom_uint16_t" #-} c'hkl_binoculars_space_qcustom_uint16_t pSpace geometry i nPixels (CDouble . unAttenuation $ att) pix (toEnum ndim) dims r (toEnum nr) mask'' (toEnum $ fromEnum surf) limits (toEnum nlimits) (CDouble . unTimestamp $ index) (toEnum . fromEnum $ subprojection) (CDouble (uqx /~ radian)) (CDouble (uqy /~ radian)) (CDouble (uqz /~ radian)) sampleAxis (toEnum . fromEnum $ doPolarizationCorrection)
+      {-# SCC "hkl_binoculars_space_qcustom_uint16_t" #-} c'hkl_binoculars_space_qcustom_uint16_t pSpace geometry i nPixels (CDouble . unAttenuation $ att) pix (toEnum ndim) dims r (toEnum nr) mask'' (toEnum $ fromEnum surf) limits (toEnum nlimits) (CDouble . unTimestamp $ index) (CDouble . unTimescan0 $ timescan0) (toEnum . fromEnum $ subprojection) (CDouble (uqx /~ radian)) (CDouble (uqy /~ radian)) (CDouble (uqz /~ radian)) sampleAxis (toEnum . fromEnum $ doPolarizationCorrection)
     (ImageWord32 arr) -> unsafeWith arr $ \i -> do
-      {-# SCC "hkl_binoculars_space_qcustom_uint32_t" #-} c'hkl_binoculars_space_qcustom_uint32_t pSpace geometry i nPixels (CDouble . unAttenuation $ att) pix (toEnum ndim) dims r (toEnum nr) mask'' (toEnum $ fromEnum surf) limits (toEnum nlimits) (CDouble . unTimestamp $ index) (toEnum . fromEnum $ subprojection) (CDouble (uqx /~ radian)) (CDouble (uqy /~ radian)) (CDouble (uqz /~ radian)) sampleAxis (toEnum . fromEnum $ doPolarizationCorrection)
+      {-# SCC "hkl_binoculars_space_qcustom_uint32_t" #-} c'hkl_binoculars_space_qcustom_uint32_t pSpace geometry i nPixels (CDouble . unAttenuation $ att) pix (toEnum ndim) dims r (toEnum nr) mask'' (toEnum $ fromEnum surf) limits (toEnum nlimits) (CDouble . unTimestamp $ index) (CDouble . unTimescan0 $ timescan0) (toEnum . fromEnum $ subprojection) (CDouble (uqx /~ radian)) (CDouble (uqy /~ radian)) (CDouble (uqz /~ radian)) sampleAxis (toEnum . fromEnum $ doPolarizationCorrection)
 
   return (DataFrameSpace img space att)
 
@@ -900,7 +957,7 @@ processQCustomP = do
                              each job
                              >-> Pipes.Prelude.map (\(Chunk fn f t) -> (fn, [f..t]))
                              >-> framesP datapaths
-                             >-> Pipes.Prelude.filter (\(DataFrameQCustom _ _ img _) -> filterSumImage mImageSumMax img)
+                             >-> Pipes.Prelude.filter (\(DataFrameQCustom _ _ img _ _) -> filterSumImage mImageSumMax img)
                              >-> project det 3 (spaceQCustom det pixels res mask' surfaceOrientation mlimits subprojection uqx uqy uqz mSampleAxis doPolarizationCorrection)
                              >-> tee (accumulateP c)
                              >-> progress pb
@@ -909,7 +966,7 @@ processQCustomP = do
 
 
 instance ChunkP (DataSourcePath DataFrameQCustom) where
-    chunkP mSkipFirst mSkipLast (DataSourcePath'DataFrameQCustom ma _ (DataSourcePath'Image i _) _) =
+    chunkP mSkipFirst mSkipLast (DataSourcePath'DataFrameQCustom ma _ (DataSourcePath'Image i _) _ _) =
       skipMalformed $ forever $ do
       fp <- await
       withFileP (openFile' fp) $ \f ->
