@@ -13,7 +13,7 @@
  * You should have received a copy of the GNU General Public License
  * along with the hkl library.  If not, see <http://www.gnu.org/licenses/>.
  *
- * Copyright (C) 2003-2018 Synchrotron SOLEIL
+ * Copyright (C) 2003-2019, 2023 Synchrotron SOLEIL
  *                         L'Orme des Merisiers Saint-Aubin
  *                         BP 48 91192 GIF-sur-YVETTE CEDEX
  *
@@ -151,12 +151,45 @@ HKLAPI void hkl_matrix_times_matrix(HklMatrix *self, const HklMatrix *m) HKL_ARG
 /* Unit */
 /********/
 
+typedef struct _HklUnitDimension HklUnitDimension;
+
+struct _HklUnitDimension
+{
+	int l; /* Length */
+	int m; /* Mass */
+	int t; /* Time */
+	int i; /* Electric current */
+	int th; /* Thermodynamic temperature */
+	int n; /* Amount of substance */
+	int j; /* Luminous intensity */
+};
+
+#define HklDPlaneAngle {0, 0, 0, 0, 0, 0, 0}
+#define HklDLength {1, 0, 0, 0, 0, 0, 0}
+
 typedef enum _HklUnitEnum
 {
 	HKL_UNIT_DEFAULT,
 	HKL_UNIT_USER
 } HklUnitEnum;
 
+typedef struct _HklUnit HklUnit;
+
+struct _HklUnit
+{
+	HklUnitDimension dimension;
+	double factor;
+	char const *name;
+	char const *repr;
+};
+
+extern const HklUnit hkl_unit_angle_deg;
+extern const HklUnit hkl_unit_angle_rad;
+extern const HklUnit hkl_unit_length_nm;
+extern const HklUnit hkl_unit_angle_mrad;
+extern const HklUnit hkl_unit_length_mm;
+extern const HklUnit hkl_unit_length_meter;
+
 /*************/
 /* Parameter */
 /*************/
@@ -276,6 +309,19 @@ HKLAPI HklQuaternion hkl_geometry_sample_rotation_get(const HklGeometry *self,
 HKLAPI HklQuaternion hkl_geometry_detector_rotation_get(const HklGeometry *self,
 							const HklDetector *detector) HKL_ARG_NONNULL(1, 2);
 
+HKLAPI HklVector hkl_geometry_ki_get(const HklGeometry *self) HKL_ARG_NONNULL(1);
+
+HKLAPI HklVector hkl_geometry_ki_abc_get(const HklGeometry *self,
+                                         const HklSample *sample) HKL_ARG_NONNULL(1, 2);
+
+
+HKLAPI HklVector hkl_geometry_kf_get(const HklGeometry *self,
+				     const HklDetector *detector) HKL_ARG_NONNULL(1, 2);
+
+HKLAPI HklVector hkl_geometry_kf_abc_get(const HklGeometry *self,
+                                         const HklDetector *detector,
+                                         const HklSample *sample) HKL_ARG_NONNULL(1, 2, 3);
+
 HKLAPI void hkl_geometry_fprintf(FILE *file, const HklGeometry *self) HKL_ARG_NONNULL(1, 2);
 
 /* HklGeometryList */
@@ -572,6 +618,23 @@ HKLAPI void hkl_engine_list_init(HklEngineList *self,
 
 HKLAPI int hkl_engine_list_get(HklEngineList *self) HKL_ARG_NONNULL(1);
 
+HKLAPI const darray_string *hkl_engine_list_parameters_names_get(const HklEngineList *self) HKL_ARG_NONNULL(1) HKL_WARN_UNUSED_RESULT;
+
+HKLAPI const HklParameter *hkl_engine_list_parameter_get(const HklEngineList *self, const char *name,
+							 GError **error) HKL_ARG_NONNULL(1, 2) HKL_WARN_UNUSED_RESULT;
+
+HKLAPI int hkl_engine_list_parameter_set(HklEngineList *self,
+					 const char *name, const HklParameter *parameter,
+					 GError **error) HKL_ARG_NONNULL(1, 2, 3) HKL_WARN_UNUSED_RESULT;
+
+HKLAPI void hkl_engine_list_parameters_values_get(const HklEngineList *self,
+						  double values[], size_t n_values,
+						  HklUnitEnum unit_type) HKL_ARG_NONNULL(1, 2);
+
+HKLAPI int hkl_engine_list_parameters_values_set(HklEngineList *self,
+						 double values[], size_t n_values,
+						 HklUnitEnum unit_type, GError **error) HKL_ARG_NONNULL(1, 2) HKL_WARN_UNUSED_RESULT;
+
 HKLAPI void hkl_engine_list_fprintf(FILE *f,
 				    const HklEngineList *self) HKL_ARG_NONNULL(1, 2);
 
